import 'dart:math';

import 'package:prefix_sum/prefix_sum.dart';
import 'package:test/test.dart';

void main() {
  test('should act like an ordinary List<num>', () {
    expect(SparsePrefixSumList(1) is List<num>, true);

    final random = Random();
    final length = random.nextInt(10) + 1;
    final defaultValue = random.nextInt(100);
    final list = SparsePrefixSumList(defaultValue, length);

    for (int i = 0; i < length; i++) {
      expect(list[i], defaultValue);
    }
    for (int i = 0; i < 10; i++) {
      final newValue = random.nextInt(100);
      final newIndex = random.nextInt(length);
      list[newIndex] = newValue;
      expect(list[newIndex], newValue);
    }
  });

  group('should contain a prefix sum index', () {
    final random = Random();
    final values = Map<int, num>();
    var length = random.nextInt(1000);
    var defaultValue = 100;

    // fill values
    for (int i = 0; i < random.nextInt((0.5 * length).floor()); i++) {
      values[random.nextInt(length)] = random.nextInt(1000);
    }

    final list = SparsePrefixSumList(defaultValue, length);
    values.forEach((k, v) => list[k] = v);
    values.forEach((k, v) => list[k] += v);

    test('should be able to get sum from index', () {
      for (int i = 0; i < length; i++) {
        final realSum = list.getPrefixSum(i);
        final expectSum = list.sublist(0, i + 1).reduce((a, b) => a + b);
        expect(realSum, expectSum);
      }
    });

    test('should be able to find index from sum', () {
      for (int i = 0; i < length; i++) {
        final sum = list.sublist(0, i + 1).reduce((a, b) => a + b) +
            random.nextDouble();
        final realIndex = list.findIndexBeforePrefixSum(sum);
        expect(realIndex, i);
      }
    });

    test('should be able to update default value', () {
      defaultValue = random.nextInt(100);
      for (int i = 0; i < length; i++) {
        final realSum = list.getPrefixSum(i);
        final expectSum = list.sublist(0, i + 1).reduce((a, b) => a + b);
        expect(realSum, expectSum);
      }
      for (int i = 0; i < length; i++) {
        final sum = list.sublist(0, i + 1).reduce((a, b) => a + b) +
            random.nextDouble();
        final realIndex = list.findIndexBeforePrefixSum(sum);
        expect(realIndex, i);
      }
    });

    test('should be growable', () {
      final list = SparsePrefixSumList(50);
      list[0] = 200;
      expect(list.getPrefixSum(0), 200);
      list[1] = 200;
      expect(list.getPrefixSum(1), 400);
      list[2] = 200;
      expect(list.getPrefixSum(2), 600);
      list[3] = 200;
      expect(list.getPrefixSum(3), 800);
      list[4] = 200;
      expect(list.getPrefixSum(4), 1000);
      list[5] = 200;
      expect(list.getPrefixSum(5), 1200);
      list[6] = 200;
      expect(list.getPrefixSum(6), 1400);
      list[7] = 200;
      expect(list.getPrefixSum(7), 1600);
      list[8] = 200;
      expect(list.getPrefixSum(8), 1800);
    });
  });
}

List<num> sumList(List<num> from) {
  final sumList = <num>[from[0]];
  from.reduce((a, b) {
    final next = a + b;
    sumList.add(next);
    return next;
  });
  return sumList;
}
